@@ -11,11 +11,16 @@
 # You should have received a copy of the GNU General Public License along with django-ca.  If not,
 # see <http://www.gnu.org/licenses/>.
 
+"""Management command to write an OCSP index to stdout or a file.
+
+.. seealso:: https://docs.djangoproject.com/en/dev/howto/custom-management-commands/
+"""
+
 from ...ocsp import get_index
 from ..base import BaseCommand
 
 
-class Command(BaseCommand):
+class Command(BaseCommand):  # pylint: disable=missing-class-docstring
     help = "Write an OCSP index file."
 
     def add_arguments(self, parser):
@@ -23,7 +28,7 @@ class Command(BaseCommand):
         parser.add_argument('path', type=str, default='-', nargs='?',
                             help="Where to write the index (default: stdout)")
 
-    def handle(self, ca, path, **options):
+    def handle(self, ca, path, **options):  # pylint: disable=arguments-differ
         if path == '-':
             for line in get_index(ca):
                 self.stdout.write(line)
