@@ -11,9 +11,9 @@
 # You should have received a copy of the GNU General Public License along with django-ca.  If not,
 # see <http://www.gnu.org/licenses/>.
 
-"""
-Inspired by:
-https://skippylovesmalorie.wordpress.com/2010/02/12/how-to-generate-a-self-signed-certificate-using-pyopenssl/
+"""Management command to create a certificate authority.
+
+.. seealso:: https://docs.djangoproject.com/en/dev/howto/custom-management-commands/
 """
 
 import os
@@ -38,10 +38,12 @@ from ..base import PasswordAction
 from ..base import URLAction
 
 
-class Command(BaseCommand, CertificateAuthorityDetailMixin):
+class Command(BaseCommand, CertificateAuthorityDetailMixin):  # pylint: disable=missing-class-docstring
+    """Create a certificate authority."""
     help = "Create a certificate authority."
 
     def add_arguments(self, parser):
+        self.add_general_args(parser)
         self.add_algorithm(parser)
 
         self.add_key_type(parser)
@@ -83,6 +85,8 @@ class Command(BaseCommand, CertificateAuthorityDetailMixin):
         group.add_argument('--no-default-hostname', dest='default_hostname', action='store_false',
                            help='Disable any default hostname configured in your settings.')
 
+        self.add_acme_group(parser)
+
         group = parser.add_argument_group(
             'pathlen attribute',
             """Maximum number of CAs that can appear below this one. A pathlen of zero (the default) means it
@@ -124,7 +128,7 @@ class Command(BaseCommand, CertificateAuthorityDetailMixin):
 
         self.add_ca_args(parser)
 
-    def handle(self, name, subject, **options):
+    def handle(self, name, subject, **options):  # pylint: disable=arguments-differ
         if not os.path.exists(ca_settings.CA_DIR):  # pragma: no cover
             # TODO: set permissions
             os.makedirs(ca_settings.CA_DIR)
@@ -168,6 +172,12 @@ class Command(BaseCommand, CertificateAuthorityDetailMixin):
             if options[opt] is not None:
                 kwargs[opt] = options[opt]
 
+        if ca_settings.CA_ENABLE_ACME:  # pragma: no branch; never False because parser throws error already
+            # These settings are only there if ACME is enabled
+            for opt in ['acme_enabled', 'acme_requires_contact']:
+                if options[opt] is not None:
+                    kwargs[opt] = options[opt]
+
         try:
             ca = CertificateAuthority.objects.init(
                 key_size=options['key_size'], key_type=options['key_type'],
@@ -176,7 +186,7 @@ class Command(BaseCommand, CertificateAuthorityDetailMixin):
                 expires=options['expires'],
                 pathlen=options['pathlen'],
                 issuer_url=options['issuer_url'],
-                issuer_alt_name=issuer_alternative_name,
+                issuer_alt_name=','.join(issuer_alternative_name),
                 crl_url=options['crl_url'],
                 ocsp_url=options['ocsp_url'],
                 ca_issuer_url=options['ca_issuer_url'],
@@ -185,10 +195,13 @@ class Command(BaseCommand, CertificateAuthorityDetailMixin):
                 name_constraints=name_constraints,
                 name=name, subject=subject, password=options['password'],
                 parent_password=options['parent_password'],
+                caa=options['caa'],
+                website=options['website'],
+                terms_of_service=options['tos'],
                 **kwargs
             )
-        except Exception as e:
-            raise CommandError(e)
+        except Exception as ex:
+            raise CommandError(ex) from ex
 
         # Generate OCSP keys and cache CRLs
         run_task(generate_ocsp_key, serial=ca.serial, password=options['password'])
