@@ -11,6 +11,8 @@
 # You should have received a copy of the GNU General Public License along with django-ca. If not,
 # see <http://www.gnu.org/licenses/>.
 
+"""Django model managers."""
+
 import pathlib
 
 from cryptography import x509
@@ -45,8 +47,13 @@ from .utils import validate_hostname
 from .utils import validate_key_parameters
 
 
-class CertificateManagerMixin(object):
+class CertificateManagerMixin:  # pylint: disable=too-few-public-methods
+    """Mixin for model managers."""
+
     def get_common_extensions(self, issuer_url=None, crl_url=None, ocsp_url=None):
+        """Add extensions potentially common to both CAs and certs."""
+        # pylint: disable=no-self-use
+
         extensions = []
         if crl_url:
             urls = [x509.UniformResourceIdentifier(force_str(c)) for c in crl_url]
@@ -66,7 +73,7 @@ class CertificateManagerMixin(object):
             extensions.append((False, x509.AuthorityInformationAccess(auth_info_access)))
         return extensions
 
-    def _extra_extensions(self, builder, extra_extensions):
+    def _extra_extensions(self, builder, extra_extensions):  # pylint: disable=no-self-use
         for ext in extra_extensions:
             if isinstance(ext, x509.extensions.Extension):
                 builder = builder.add_extension(ext.value, critical=ext.critical)
@@ -78,11 +85,14 @@ class CertificateManagerMixin(object):
 
 
 class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
+    """Model manager for the CertificateAuthority model."""
+
     def init(self, name, subject, expires=None, algorithm=None, parent=None, default_hostname=None,
              pathlen=None, issuer_url=None, issuer_alt_name='', crl_url=None, ocsp_url=None,
              ca_issuer_url=None, ca_crl_url=None, ca_ocsp_url=None, name_constraints=None,
              password=None, parent_password=None, ecc_curve=None, key_type='RSA', key_size=None,
-             extra_extensions=None, path='ca'):
+             extra_extensions=None, path='ca',
+             caa='', website='', terms_of_service='', acme_enabled=False, acme_requires_contact=True):
         """Create a new certificate authority.
 
         Parameters
@@ -151,6 +161,16 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
             An optional list of additional extensions to add to the certificate.
         path : str or pathlib.PurePath, optional
             Where to store the CA private key (default ``ca``).
+        caa : str, optional
+            CAA identity. Note that this field is not yet currently used.
+        website : str, optional
+            URL to a human-readable website.
+        terms_of_service : str, optional
+            URL to the terms of service for the CA.
+        acme_enabled : bool, optional
+            Set to ``True`` to enable ACME support for this CA.
+        acme_requires_contact : bool, optional
+            Set to ``False`` if you do not want to force clients to register with an email address.
 
         Raises
         ------
@@ -160,6 +180,7 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
         PermissionError
             If the private key file cannot be written to disk.
         """
+        # pylint: disable=too-many-arguments,too-many-locals,too-many-branches,too-many-statements
         # NOTE: Already verified by KeySizeAction, so these checks are only for when the Python API is used
         #       directly.
         key_size, key_type, ecc_curve = validate_key_parameters(key_size, key_type, ecc_curve)
@@ -169,8 +190,7 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
         if not isinstance(subject, Subject):
             subject = Subject(subject)
         if issuer_alt_name and not isinstance(issuer_alt_name, IssuerAlternativeName):
-            issuer_alt_name = IssuerAlternativeName(issuer_alt_name)
-            issuer_alt_name = ','.join(issuer_alt_name.serialize()['value'])
+            issuer_alt_name = IssuerAlternativeName({'value': [issuer_alt_name]})
         if crl_url is None:
             crl_url = []
 
@@ -184,7 +204,7 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
         if default_hostname:
             default_hostname = validate_hostname(default_hostname, allow_port=True)
             if parent:
-                root_serial = parent.root.serial
+                root_serial = parent.serial
             else:
                 root_serial = hex_serial
 
@@ -193,7 +213,7 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
                 ocsp_path = reverse('django_ca:ocsp-cert-post', kwargs={'serial': hex_serial})
                 ocsp_url = 'http://%s%s' % (default_hostname, ocsp_path)
             if parent and not ca_ocsp_url:  # OCSP for CA only makes sense in intermediate CAs
-                ocsp_path = reverse('django_ca:ocsp-ca-post', kwargs={'serial': hex_serial})
+                ocsp_path = reverse('django_ca:ocsp-ca-post', kwargs={'serial': root_serial})
                 ca_ocsp_url = 'http://%s%s' % (default_hostname, ocsp_path)
 
             # Set issuer path
@@ -208,7 +228,7 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
                 crl_path = reverse('django_ca:crl', kwargs={'serial': hex_serial})
                 crl_url = ['http://%s%s' % (default_hostname, crl_path)]
             if parent and not ca_crl_url:  # CRL for CA only makes sense in intermediate CAs
-                ca_crl_path = reverse('django_ca:ca-crl', kwargs={'serial': hex_serial})
+                ca_crl_path = reverse('django_ca:ca-crl', kwargs={'serial': root_serial})
                 ca_crl_url = ['http://%s%s' % (default_hostname, ca_crl_path)]
 
         pre_create_ca.send(
@@ -216,7 +236,9 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
             expires=expires, parent=parent, subject=subject, pathlen=pathlen, issuer_url=issuer_url,
             issuer_alt_name=issuer_alt_name, crl_url=crl_url, ocsp_url=ocsp_url, ca_issuer_url=ca_issuer_url,
             ca_crl_url=ca_crl_url, ca_ocsp_url=ca_ocsp_url, name_constraints=name_constraints,
-            password=password, parent_password=parent_password, extra_extensions=extra_extensions)
+            password=password, parent_password=parent_password, extra_extensions=extra_extensions, caa=caa,
+            website=website, terms_of_service=terms_of_service, acme_enabled=acme_enabled,
+            acme_requires_contact=acme_requires_contact)
 
         private_key = generate_private_key(key_size, key_type, ecc_curve)
         public_key = private_key.public_key()
@@ -263,8 +285,10 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
         # Normalize extensions for create()
         crl_url = '\n'.join(crl_url)
 
-        ca = self.model(name=name, issuer_url=issuer_url, issuer_alt_name=issuer_alt_name,
-                        ocsp_url=ocsp_url, crl_url=crl_url, parent=parent)
+        ca = self.model(name=name, issuer_url=issuer_url, issuer_alt_name=','.join(issuer_alt_name),
+                        ocsp_url=ocsp_url, crl_url=crl_url, parent=parent, caa_identity=caa, website=website,
+                        terms_of_service=terms_of_service, acme_enabled=acme_enabled,
+                        acme_requires_contact=acme_requires_contact)
         ca.x509 = certificate
 
         if password is None:
@@ -285,12 +309,16 @@ class CertificateAuthorityManager(CertificateManagerMixin, models.Manager):
 
 
 class CertificateManager(CertificateManagerMixin, models.Manager):
-    def parse_csr(self, csr, csr_format):
+    """Model manager for the Certificate model."""
+
+    def parse_csr(self, csr, csr_format):  # pylint: disable=no-self-use; TODO: move to utils
+        """Parse a CSR in the given format."""
+
         if isinstance(csr, x509.CertificateSigningRequest):
             return csr
-        elif csr_format == Encoding.PEM:
+        if csr_format == Encoding.PEM:
             return x509.load_pem_x509_csr(force_bytes(csr), default_backend())
-        elif csr_format == Encoding.DER:
+        if csr_format == Encoding.DER:
             return x509.load_der_x509_csr(force_bytes(csr), default_backend())
 
         raise ValueError('Unknown CSR format passed: %s' % csr_format)
@@ -325,14 +353,34 @@ class CertificateManager(CertificateManagerMixin, models.Manager):
         csr = self.parse_csr(csr, csr_format=csr_format)
         cert = profile.create_cert(ca, csr, **kwargs)
 
-        c = self.model(ca=ca, csr=csr.public_bytes(Encoding.PEM).decode('utf-8'), profile=profile.name)
-        c.x509 = cert
+        obj = self.model(ca=ca, csr=csr.public_bytes(Encoding.PEM).decode('utf-8'), profile=profile.name)
+        obj.x509 = cert
         if autogenerated is None:
-            c.autogenerated = profile.autogenerated
+            obj.autogenerated = profile.autogenerated
         else:
-            c.autogenerated = autogenerated
-        c.save()
+            obj.autogenerated = autogenerated
+        obj.save()
 
-        post_issue_cert.send(sender=self.model, cert=c)
+        post_issue_cert.send(sender=self.model, cert=obj)
 
-        return c
+        return obj
+
+
+class AcmeAccountManager(models.Manager):
+    """Model manager for :py:class:`~django_ca.models.AcmeAccount`."""
+
+
+class AcmeOrderManager(models.Manager):
+    """Model manager for :py:class:`~django_ca.models.AcmeOrder`."""
+
+
+class AcmeAuthorizationManager(models.Manager):
+    """Model manager for :py:class:`~django_ca.models.AcmeAuthorization`."""
+
+
+class AcmeChallengeManager(models.Manager):
+    """Model manager for :py:class:`~django_ca.models.AcmeChallenge`."""
+
+
+class AcmeCertificateManager(models.Manager):
+    """Model manager for :py:class:`~django_ca.models.AcmeCertificate`."""
